@@ -13,7 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import base64
 import gzip
 import logging
 import math
@@ -28,6 +27,7 @@ import time
 
 from oslo_concurrency import processutils
 from oslo_config import cfg
+from oslo_serialization import base64
 from oslo_utils import excutils
 from oslo_utils import imageutils
 from oslo_utils import units
@@ -356,14 +356,15 @@ def destroy_disk_metadata(dev, node_uuid):
                       run_as_root=True,
                       use_standard_locale=True)
     except processutils.ProcessExecutionError as e:
-        # NOTE(zhenguo): Check if --force option is supported for wipefs,
-        # if not, we should try without it.
-        if '--force' in str(e):
-            utils.execute('wipefs', '--all', dev,
-                          run_as_root=True,
-                          use_standard_locale=True)
-        else:
-            raise e
+        with excutils.save_and_reraise_exception() as ctxt:
+            # NOTE(zhenguo): Check if --force option is supported for wipefs,
+            # if not, we should try without it.
+            if '--force' in str(e):
+                ctxt.reraise = False
+                utils.execute('wipefs', '--all', dev,
+                              run_as_root=True,
+                              use_standard_locale=True)
+
     LOG.info(_LI("Disk metadata on %(dev)s successfully destroyed for node "
                  "%(node)s"), {'dev': dev, 'node': node_uuid})
 
@@ -395,7 +396,7 @@ def _get_configdrive(configdrive, node_uuid, tempdir=None):
         data = configdrive
 
     try:
-        data = six.BytesIO(base64.b64decode(data))
+        data = six.BytesIO(base64.decode_as_bytes(data))
     except TypeError:
         error_msg = (_('Config drive for node %s is not base64 encoded '
                        'or the content is malformed.') % node_uuid)
