@@ -13,7 +13,6 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import base64
 import gzip
 import mock
 import os
@@ -21,9 +20,9 @@ import shutil
 import stat
 import tempfile
 
-
 from oslo_concurrency import processutils
 from oslo_config import cfg
+from oslo_serialization import base64
 from oslo_service import loopingcall
 from oslo_utils import imageutils
 from oslotest import base as test_base
@@ -228,6 +227,71 @@ class WorkOnDiskTestCase(test_base.BaseTestCase):
                                              boot_mode="bios",
                                              disk_label='gpt')
 
+    @mock.patch.object(disk_utils, 'block_uuid', autospec=True)
+    @mock.patch.object(disk_utils, 'populate_image', autospec=True)
+    @mock.patch.object(utils, 'mkfs', autospec=True)
+    def test_uefi_localboot(self, mock_mkfs, mock_populate_image,
+                            mock_block_uuid):
+        """Test that we create a fat filesystem with UEFI localboot."""
+        root_part = '/dev/fake-part1'
+        efi_part = '/dev/fake-part2'
+        self.mock_mp.return_value = {'root': root_part,
+                                     'efi system partition': efi_part}
+        self.mock_ibd.return_value = True
+        mock_ibd_calls = [mock.call(root_part),
+                          mock.call(efi_part)]
+
+        disk_utils.work_on_disk(self.dev, self.root_mb,
+                                self.swap_mb, self.ephemeral_mb,
+                                self.ephemeral_format,
+                                self.image_path, self.node_uuid,
+                                boot_option="local", boot_mode="uefi")
+
+        self.mock_mp.assert_called_once_with(self.dev, self.root_mb,
+                                             self.swap_mb, self.ephemeral_mb,
+                                             self.configdrive_mb,
+                                             self.node_uuid, commit=True,
+                                             boot_option="local",
+                                             boot_mode="uefi",
+                                             disk_label=None)
+        self.assertEqual(self.mock_ibd.call_args_list, mock_ibd_calls)
+        mock_mkfs.assert_called_once_with(fs='vfat', path=efi_part,
+                                          label='efi-part')
+        mock_populate_image.assert_called_once_with(self.image_path,
+                                                    root_part)
+        mock_block_uuid.assert_any_call(root_part)
+        mock_block_uuid.assert_any_call(efi_part)
+
+    @mock.patch.object(disk_utils, 'block_uuid', autospec=True)
+    @mock.patch.object(disk_utils, 'populate_image', autospec=True)
+    @mock.patch.object(utils, 'mkfs', autospec=True)
+    def test_preserve_ephemeral(self, mock_mkfs, mock_populate_image,
+                                mock_block_uuid):
+        """Test that ephemeral partition doesn't get overwritten."""
+        ephemeral_part = '/dev/fake-part1'
+        root_part = '/dev/fake-part2'
+        ephemeral_mb = 256
+        ephemeral_format = 'exttest'
+
+        self.mock_mp.return_value = {'ephemeral': ephemeral_part,
+                                     'root': root_part}
+        self.mock_ibd.return_value = True
+        calls = [mock.call(root_part),
+                 mock.call(ephemeral_part)]
+        disk_utils.work_on_disk(self.dev, self.root_mb,
+                                self.swap_mb, ephemeral_mb, ephemeral_format,
+                                self.image_path, self.node_uuid,
+                                preserve_ephemeral=True)
+        self.assertEqual(self.mock_ibd.call_args_list, calls)
+        self.mock_mp.assert_called_once_with(self.dev, self.root_mb,
+                                             self.swap_mb, ephemeral_mb,
+                                             self.configdrive_mb,
+                                             self.node_uuid, commit=False,
+                                             boot_option="netboot",
+                                             boot_mode="bios",
+                                             disk_label=None)
+        self.assertFalse(mock_mkfs.called)
+
 
 @mock.patch.object(utils, 'execute', autospec=True)
 class MakePartitionsTestCase(test_base.BaseTestCase):
@@ -575,7 +639,7 @@ class GetConfigdriveTestCase(test_base.BaseTestCase):
                           'http://1.2.3.4/cd', 'fake-node-uuid')
         self.assertFalse(mock_copy.called)
 
-    @mock.patch.object(base64, 'b64decode', autospec=True)
+    @mock.patch.object(base64, 'decode_as_bytes', autospec=True)
     def test_get_configdrive_base64_error(self, mock_b64, mock_requests,
                                           mock_copy):
         mock_b64.side_effect = TypeError
@@ -672,8 +736,40 @@ class OtherFunctionTestCase(test_base.BaseTestCase):
         self.assertEqual(2, disk_utils.get_image_mb('x', False))
         self.assertEqual(2, disk_utils.get_image_mb('x', True))
 
+    def _test_count_mbr_partitions(self, output, mock_execute):
+        mock_execute.return_value = (output, '')
+        out = disk_utils.count_mbr_partitions('/dev/fake')
+        mock_execute.assert_called_once_with('partprobe', '-d', '-s',
+                                             '/dev/fake', run_as_root=True,
+                                             use_standard_locale=True)
+        return out
 
-@mock.patch.object(utils, 'execute')
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_count_mbr_partitions(self, mock_execute):
+        output = "/dev/fake: msdos partitions 1 2 3 <5 6>"
+        pp, lp = self._test_count_mbr_partitions(output, mock_execute)
+        self.assertEqual(3, pp)
+        self.assertEqual(2, lp)
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_count_mbr_partitions_no_logical_partitions(self, mock_execute):
+        output = "/dev/fake: msdos partitions 1 2"
+        pp, lp = self._test_count_mbr_partitions(output, mock_execute)
+        self.assertEqual(2, pp)
+        self.assertEqual(0, lp)
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_count_mbr_partitions_wrong_partition_table(self, mock_execute):
+        output = "/dev/fake: gpt partitions 1 2 3 4 5 6"
+        mock_execute.return_value = (output, '')
+        self.assertRaises(ValueError, disk_utils.count_mbr_partitions,
+                          '/dev/fake')
+        mock_execute.assert_called_once_with('partprobe', '-d', '-s',
+                                             '/dev/fake', run_as_root=True,
+                                             use_standard_locale=True)
+
+
+@mock.patch.object(utils, 'execute', autospec=True)
 class WholeDiskPartitionTestCases(test_base.BaseTestCase):
 
     def setUp(self):
@@ -726,7 +822,7 @@ class WholeDiskPartitionTestCases(test_base.BaseTestCase):
         ]
         mock_execute.assert_has_calls(execute_calls)
 
-    @mock.patch.object(disk_utils.LOG, 'error')
+    @mock.patch.object(disk_utils.LOG, 'error', autospec=True)
     def test_get_partition_exc(self, mock_log, mock_execute):
         mock_execute.side_effect = processutils.ProcessExecutionError
         self.assertRaisesRegex(exception.InstanceDeployFailure,
@@ -758,7 +854,7 @@ class WholeDiskPartitionTestCases(test_base.BaseTestCase):
                                                       blk_out=blkid_out)
         self.assertTrue(ret)
 
-    @mock.patch.object(disk_utils.LOG, 'error')
+    @mock.patch.object(disk_utils.LOG, 'error', autospec=True)
     def test_is_disk_larger_than_max_size_exc(self, mock_log, mock_execute):
         mock_execute.side_effect = processutils.ProcessExecutionError
         self.assertRaisesRegex(exception.InstanceDeployFailure,
@@ -792,7 +888,7 @@ class WholeDiskPartitionTestCases(test_base.BaseTestCase):
                                              use_standard_locale=True,
                                              run_as_root=True)
 
-    @mock.patch.object(disk_utils.LOG, 'error')
+    @mock.patch.object(disk_utils.LOG, 'error', autospec=True)
     def test_is_disk_gpt_partitioned_exc(self, mock_log, mock_execute):
         mock_execute.side_effect = processutils.ProcessExecutionError
         self.assertRaisesRegex(exception.InstanceDeployFailure,
@@ -831,7 +927,7 @@ or continue with the current setting?
                                              use_standard_locale=True,
                                              run_as_root=True)
 
-    @mock.patch.object(disk_utils.LOG, 'error')
+    @mock.patch.object(disk_utils.LOG, 'error', autospec=True)
     def test_fix_gpt_structs_exc(self, mock_log, mock_execute):
         mock_execute.side_effect = processutils.ProcessExecutionError
         self.assertRaisesRegex(exception.InstanceDeployFailure,
@@ -908,9 +1004,9 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                        autospec=True)
     def test_create_partition_gpt(self, mock_get_configdrive,
                                   mock_get_labelled_partition,
-                                  mock_list_partitions,
-                                  mock_is_disk_gpt, mock_fix_gpt,
-                                  mock_dd, mock_unlink, mock_execute):
+                                  mock_list_partitions, mock_is_disk_gpt,
+                                  mock_fix_gpt, mock_dd, mock_unlink,
+                                  mock_execute):
         config_url = 'http://1.2.3.4/cd'
         configdrive_file = '/tmp/xyz'
         configdrive_mb = 10
@@ -941,14 +1037,19 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
         expected_part = '/dev/fake4'
         disk_utils.create_config_drive_partition(self.node_uuid, self.dev,
                                                  config_url)
-        mock_execute.assert_called_with('sgdisk', '-n', '0:-64MB:0',
-                                        self.dev, run_as_root=True)
+        mock_execute.assert_has_calls([
+            mock.call('sgdisk', '-n', '0:-64MB:0', self.dev,
+                      run_as_root=True),
+            mock.call('udevadm', 'settle',
+                      '--exit-if-exists=%s' % expected_part),
+        ])
         self.assertEqual(2, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
         mock_fix_gpt.assert_called_with(self.dev, self.node_uuid)
         mock_dd.assert_called_with(configdrive_file, expected_part)
         mock_unlink.assert_called_with(configdrive_file)
 
+    @mock.patch.object(disk_utils, 'count_mbr_partitions', autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(disk_utils.LOG, 'warning')
     @mock.patch.object(utils, 'unlink_without_raise',
@@ -973,7 +1074,7 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                                    mock_is_disk_gpt, mock_fix_gpt,
                                    mock_disk_exceeds, mock_dd,
                                    mock_unlink, mock_log, mock_execute,
-                                   disk_size_exceeds_max=False,
+                                   mock_count, disk_size_exceeds_max=False,
                                    is_iscsi_device=False):
         config_url = 'http://1.2.3.4/cd'
         configdrive_file = '/tmp/xyz'
@@ -997,8 +1098,9 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                               {'end': 51099, 'number': 5, 'start': 49153,
                                'flags': '', 'filesystem': '', 'size': 2046}]
         mock_list_partitions.side_effect = [initial_partitions,
-                                            initial_partitions,
                                             updated_partitions]
+        # 2 primary partitions, 0 logical partitions
+        mock_count.return_value = (2, 0)
         mock_get_configdrive.return_value = (configdrive_mb, configdrive_file)
         mock_get_labelled_partition.return_value = None
         mock_is_disk_gpt.return_value = False
@@ -1016,22 +1118,28 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
         if disk_size_exceeds_max:
             self.assertEqual(1, mock_log.call_count)
-            mock_execute.assert_called_with('parted', '-a', 'optimal', '-s',
-                                            '--', self.dev, 'mkpart',
-                                            'primary', 'ext2', 2097087,
-                                            2097151, run_as_root=True)
+            parted_call = mock.call('parted', '-a', 'optimal', '-s',
+                                    '--', self.dev, 'mkpart',
+                                    'primary', 'ext2', 2097087,
+                                    2097151, run_as_root=True)
         else:
             self.assertEqual(0, mock_log.call_count)
-            mock_execute.assert_called_with('parted', '-a', 'optimal', '-s',
-                                            '--', self.dev, 'mkpart',
-                                            'primary', 'ext2', '-64MiB',
-                                            '-0', run_as_root=True)
-        self.assertEqual(3, mock_list_partitions.call_count)
+            parted_call = mock.call('parted', '-a', 'optimal', '-s',
+                                    '--', self.dev, 'mkpart',
+                                    'primary', 'ext2', '-64MiB',
+                                    '-0', run_as_root=True)
+        mock_execute.assert_has_calls([
+            parted_call,
+            mock.call('udevadm', 'settle',
+                      '--exit-if-exists=%s' % expected_part),
+        ])
+        self.assertEqual(2, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
         mock_disk_exceeds.assert_called_with(self.dev, self.node_uuid)
         mock_dd.assert_called_with(configdrive_file, expected_part)
         mock_unlink.assert_called_with(configdrive_file)
         self.assertFalse(mock_fix_gpt.called)
+        mock_count.assert_called_with(self.dev)
 
     def test__create_partition_mbr_disk_under_2TB(self):
         self._test_create_partition_mbr(disk_size_exceeds_max=False,
@@ -1041,6 +1149,7 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
         self._test_create_partition_mbr(disk_size_exceeds_max=True,
                                         is_iscsi_device=False)
 
+    @mock.patch.object(disk_utils, 'count_mbr_partitions', autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(utils, 'unlink_without_raise',
                        autospec=True)
@@ -1063,7 +1172,8 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                                                mock_list_partitions,
                                                mock_is_disk_gpt, mock_fix_gpt,
                                                mock_disk_exceeds, mock_dd,
-                                               mock_unlink, mock_execute):
+                                               mock_unlink, mock_execute,
+                                               mock_count):
         config_url = 'http://1.2.3.4/cd'
         configdrive_file = '/tmp/xyz'
         configdrive_mb = 10
@@ -1089,6 +1199,8 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
         mock_list_partitions.side_effect = [initial_partitions,
                                             initial_partitions,
                                             updated_partitions]
+        # 2 primary partitions, 0 logical partitions
+        mock_count.return_value = (2, 0)
 
         self.assertRaisesRegex(exception.InstanceDeployFailure,
                                'Disk partitioning failed on device',
@@ -1100,13 +1212,15 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                                         self.dev, 'mkpart', 'primary',
                                         'ext2', '-64MiB', '-0',
                                         run_as_root=True)
-        self.assertEqual(3, mock_list_partitions.call_count)
+        self.assertEqual(2, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
         mock_disk_exceeds.assert_called_with(self.dev, self.node_uuid)
         self.assertFalse(mock_fix_gpt.called)
         self.assertFalse(mock_dd.called)
         mock_unlink.assert_called_with(configdrive_file)
+        mock_count.assert_called_once_with(self.dev)
 
+    @mock.patch.object(disk_utils, 'count_mbr_partitions', autospec=True)
     @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(utils, 'unlink_without_raise',
                        autospec=True)
@@ -1129,7 +1243,8 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                                               mock_list_partitions,
                                               mock_is_disk_gpt, mock_fix_gpt,
                                               mock_disk_exceeds, mock_dd,
-                                              mock_unlink, mock_execute):
+                                              mock_unlink, mock_execute,
+                                              mock_count):
         config_url = 'http://1.2.3.4/cd'
         configdrive_file = '/tmp/xyz'
         configdrive_mb = 10
@@ -1147,6 +1262,8 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
         mock_disk_exceeds.return_value = False
         mock_list_partitions.side_effect = [initial_partitions,
                                             initial_partitions]
+        # 2 primary partitions, 0 logical partitions
+        mock_count.return_value = (2, 0)
 
         mock_execute.side_effect = processutils.ProcessExecutionError
 
@@ -1160,13 +1277,15 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                                         self.dev, 'mkpart', 'primary',
                                         'ext2', '-64MiB', '-0',
                                         run_as_root=True)
-        self.assertEqual(2, mock_list_partitions.call_count)
+        self.assertEqual(1, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
         mock_disk_exceeds.assert_called_with(self.dev, self.node_uuid)
         self.assertFalse(mock_fix_gpt.called)
         self.assertFalse(mock_dd.called)
         mock_unlink.assert_called_with(configdrive_file)
+        mock_count.assert_called_once_with(self.dev)
 
+    @mock.patch.object(disk_utils, 'count_mbr_partitions', autospec=True)
     @mock.patch.object(utils, 'unlink_without_raise',
                        autospec=True)
     @mock.patch.object(disk_utils, 'dd',
@@ -1185,7 +1304,8 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                                                mock_get_labelled_partition,
                                                mock_list_partitions,
                                                mock_is_disk_gpt, mock_fix_gpt,
-                                               mock_dd, mock_unlink):
+                                               mock_dd, mock_unlink,
+                                               mock_count):
         config_url = 'http://1.2.3.4/cd'
         configdrive_file = '/tmp/xyz'
         configdrive_mb = 10
@@ -1203,6 +1323,8 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
         mock_get_labelled_partition.return_value = None
         mock_is_disk_gpt.return_value = False
         mock_list_partitions.side_effect = [partitions, partitions]
+        # 4 primary partitions, 0 logical partitions
+        mock_count.return_value = (4, 0)
 
         self.assertRaisesRegex(exception.InstanceDeployFailure,
                                'Config drive cannot be created for node',
@@ -1210,11 +1332,12 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
                                self.node_uuid, self.dev, config_url)
 
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
-        self.assertEqual(2, mock_list_partitions.call_count)
+        self.assertEqual(1, mock_list_partitions.call_count)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
         self.assertFalse(mock_fix_gpt.called)
         self.assertFalse(mock_dd.called)
         mock_unlink.assert_called_with(configdrive_file)
+        mock_count.assert_called_once_with(self.dev)
 
     @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(utils, 'unlink_without_raise',
@@ -1240,3 +1363,35 @@ class WholeDiskConfigDriveTestCases(test_base.BaseTestCase):
 
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
         mock_unlink.assert_called_with(configdrive_file)
+
+    @mock.patch.object(disk_utils, 'count_mbr_partitions', autospec=True)
+    @mock.patch.object(utils, 'execute', autospec=True)
+    @mock.patch.object(utils, 'unlink_without_raise',
+                       autospec=True)
+    @mock.patch.object(disk_utils, '_is_disk_gpt_partitioned',
+                       autospec=True)
+    @mock.patch.object(disk_utils, '_get_labelled_partition',
+                       autospec=True)
+    @mock.patch.object(disk_utils, '_get_configdrive',
+                       autospec=True)
+    def test_create_partition_conf_drive_error_counting(
+            self, mock_get_configdrive, mock_get_labelled_partition,
+            mock_is_disk_gpt, mock_unlink, mock_execute, mock_count):
+        config_url = 'http://1.2.3.4/cd'
+        configdrive_file = '/tmp/xyz'
+        configdrive_mb = 10
+
+        mock_get_configdrive.return_value = (configdrive_mb, configdrive_file)
+        mock_get_labelled_partition.return_value = None
+        mock_is_disk_gpt.return_value = False
+        mock_count.side_effect = ValueError('Booooom')
+
+        self.assertRaisesRegex(exception.InstanceDeployFailure,
+                               'Failed to check the number of primary ',
+                               disk_utils.create_config_drive_partition,
+                               self.node_uuid, self.dev, config_url)
+
+        mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
+        mock_unlink.assert_called_with(configdrive_file)
+        mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
+        mock_count.assert_called_once_with(self.dev)
